from . import BasicType
from . import labeledprice


class ShippingOption(BasicType):
    fields = {
        'id': str,
        'title': str,
        'prices': {
            'class': labeledprice.LabeledPrice,
            'array': True
        }
    }

    def __init__(self, obj=None):
        super(ShippingOption, self).__init__(obj)


# Generated by Django 3.0.2

from django.db import migrations, models
import django.db.models.deletion
from .. import app_name


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TelegramChat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('telegram_id', models.CharField(max_length=128, unique=True)),
                ('type', models.CharField(choices=[('private', 'private'), ('group', 'group'), ('supergroup', 'supergroup'), ('channel', 'channel')], max_length=128)),
                ('title', models.CharField(blank=True, max_length=512, null=True)),
                ('username', models.CharField(blank=True, max_length=128, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TelegramUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('telegram_id', models.CharField(max_length=128, unique=True)),
                ('is_bot', models.BooleanField(default=False)),
                ('first_name', models.CharField(max_length=128)),
                ('last_name', models.CharField(blank=True, max_length=128, null=True)),
                ('username', models.CharField(blank=True, max_length=128, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TelegramState',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('memory', models.TextField(blank=True, null=True, verbose_name='Memory in JSON format')),
                ('name', models.CharField(blank=True, max_length=256, null=True)),
                ('telegram_chat', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telegram_states', to='{}.TelegramChat'.format(app_name))),
                ('telegram_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telegram_states', to='{}.TelegramUser'.format(app_name))),
            ],
            options={
                'unique_together': {('telegram_user', 'telegram_chat')},
            },
        ),
    ]



from . import BasicType


class PassportFile(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'file_size': int,
        'file_date': int
    }

    def __init__(self, obj=None):
        super(PassportFile, self).__init__(obj)



import os

from django.conf import settings
from django.core.management.base import BaseCommand

from django_tgbot.management import helpers


class Command(BaseCommand):
    help = 'Updates the token for an existing tgbot'

    def handle(self, *args, **options):
        bot_username = input('Enter the username of the bot (without @): ').lower()
        dst = os.path.join(settings.BASE_DIR, bot_username)
        if not os.path.isdir(dst):
            self.stdout.write(
                self.style.ERROR('No such bot found. Make sure you have created your bot with command `createtgbot`.')
            )
            return

        get_me_result, bot_token = helpers.prompt_token(self, prompt_message='Enter the new token for this bot: ')
        bot_name = str(get_me_result.get_first_name())
        received_username = str(get_me_result.get_username()).lower()

        if received_username != bot_username:
            self.stdout.write(self.style.ERROR('Given token does not belong to this bot.'))
            return

        with open(os.path.join(dst, '__init__.py'), 'w') as f:
            f.write("# Do not remove these 2 lines:\nbot_token = '{}'\napp_name='{}'\n".format(bot_token, bot_username))

        self.stdout.write(self.style.SUCCESS('Successfully updated token for bot {}(@{}).'.format(bot_name, bot_username)))



from . import BasicType
from . import photosize, maskposition


class Sticker(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'width': int,
        'height': int,
        'is_animated': BasicType.bool_interpreter,
        'thumb': photosize.PhotoSize,
        'emoji': str,
        'set_name': str,
        'mask_position': maskposition.MaskPosition,
        'file_size': int
    }

    def __init__(self, obj=None):
        super(Sticker, self).__init__(obj)




from . import BasicType


class PhotoSize(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'width': int,
        'height': int,
        'file_size': int
    }

    def __init__(self, obj=None):
        super(PhotoSize, self).__init__(obj)

    def get_file_id(self) -> str:
        return getattr(self, 'file_id', None)



import json


def validate_type_object(obj):
    if obj is None:
        return {}
    if type(obj) == dict:
        return obj

    if type(obj) == str:
        try:
            return json.loads(obj)
        except ValueError:
            return {}

    return {}


def parse_field_configured(obj, config):
    """
    Parses an object to a Telegram Type based on the configuration given
    :param obj: The object to parse
    :param config: The configuration:
        - is array?
        - is array in array?
        - type of the class to be loaded to
    :return: the parsed object
    """
    foreign_type = config if type(config) != dict else config['class']
    if type(config) == dict:

        if 'array' in config and config['array'] is True:
            return [foreign_type(x) for x in obj]

        elif 'array_of_array' in config and config['array_of_array'] is True:
            res = []
            for inner_obj in obj:
                res.append([foreign_type(x) for x in inner_obj])
            return res

    return foreign_type(obj)


class BasicType:
    """
    Types should have an attribute named `fields` with this format in order to be parsed:
    fields = {
        'key': SomeType (Message, str, ...),
        'key': {
            'class': SomeType (Message, str, ...),
            'array': True/False,
            'array_of_array': True/False,
            'validation': True/False
        }
    }
    """
    def __init__(self, obj=None):
        self.obj = validate_type_object(obj)
        self.parse_fields()

    def parse_fields(self):
        if self.obj is None:
            return
        for key in self.fields.keys():
            if key in self.obj:
                try:
                    setattr(
                        self,
                        key,
                        parse_field_configured(self.obj[key], self.fields[key])
                    )
                except ValueError:
                    pass

    def to_dict(self):
        result = {}
        for field in self.fields.keys():
            if not hasattr(self, field):
                continue
            if not hasattr(getattr(self, field), 'to_dict'):
                if type(getattr(self, field)) == list:
                    result[field] = self.make_primitive(getattr(self, field))
                else:
                    result[field] = getattr(self, field)
            else:
                result[field] = getattr(self, field).to_dict()
        return result

    @staticmethod
    def make_primitive(obj):
        if hasattr(obj, 'to_dict'):
            return obj.to_dict()

        if type(obj) == list:
            return [BasicType.make_primitive(x) for x in obj]

        return {}

    def to_json(self):
        return json.dumps(self.to_dict())

    @staticmethod
    def bool_interpreter(x):
        return True if x in [True, 'true', 'True', 1, 'T', 't', '1'] else False

    @classmethod
    def a(child_class, **kwargs):
        if not hasattr(child_class, 'fields'):
            return

        result = child_class()

        for key in child_class.fields.keys():
            if key not in kwargs or kwargs[key] is None:
                continue

            # Validating the argument
            if type(child_class.fields[key]) != dict \
                    or 'validation' not in child_class.fields[key] \
                    or child_class.fields[key]['validation']:

                if type(child_class.fields[key]) == dict:
                    field = child_class.fields[key]
                    if ('array' in field and field['array']) or ('array_of_array' in field and field['array_of_array']):
                        expected_type = list
                    else:
                        expected_type = child_class.fields[key]['class']
                else:
                    expected_type = child_class.fields[key]

                if expected_type == BasicType.bool_interpreter:
                    expected_type = bool

                if type(kwargs[key]) != expected_type:
                    raise ValueError('Type mismatch for argument {}.\nExpected type: {}\nActual type: {}'.format(
                        key, expected_type, type(kwargs[key])
                    ))

            setattr(result, key, kwargs[key])

        return result






from typing import Optional
from . import BasicType


class Chat(BasicType):
    fields = {
        'id': str,
        'type': str,
        'title': str,
        'username': str,
        'first_name': str,
        'last_name': str,
        'description': str,
        'invite_link': str,
        'slow_mode_delay': str,
        'sticker_set_name': str
    }

    def __init__(self, obj=None):
        super(Chat, self).__init__(obj)

    def get_id(self) -> str:
        return getattr(self, 'id', None)

    def get_type(self) -> str:
        return getattr(self, 'type', None)

    def get_title(self) -> Optional[str]:
        return getattr(self, 'title', None)

    def get_username(self) -> Optional[str]:
        return getattr(self, 'username', None)

    def get_first_name(self) -> Optional[str]:
        return getattr(self, 'first_name', None)

    def get_last_name(self):
        return getattr(self, 'last_name', None)

    def get_photo(self):
        return getattr(self, 'photo', None)


# These are placed here to avoid import cycles

from . import chatphoto, message, chatpermissions

Chat.fields.update({
    'pinned_message': message.Message,
    'can_set_sticker_set': BasicType.bool_interpreter,
    'photo': chatphoto.ChatPhoto,
    'permissions': chatpermissions.ChatPermissions
})



from typing import Optional

from . import BasicType


class OrderInfo(BasicType):
    fields = {
        'name': str,
        'phone_number': str,
        'email': str,
    }

    def __init__(self, obj=None):
        super(OrderInfo, self).__init__(obj)

    def get_name(self) -> Optional[str]:
        return getattr(self, 'name', None)

    def get_shipping_address(self):  # -> Optional[shippingaddress.ShippingAddress]:
        return getattr(self, 'shipping_address')


from . import shippingaddress

OrderInfo.fields.update({
    'shipping_address': shippingaddress.ShippingAddress
})


from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
import json
from django.db import models
from django.db.models import CASCADE


class AbstractTelegramChat(models.Model):
    """
    Represents a `chat` type in Bot API:
        https://core.telegram.org/bots/api#chat
    """
    CHAT_TYPES = (
        ("private", "private"),
        ("group", "group"),
        ("supergroup", "supergroup"),
        ("channel", "channel")
    )

    telegram_id = models.CharField(max_length=128, unique=True)
    type = models.CharField(choices=CHAT_TYPES, max_length=128)
    title = models.CharField(max_length=512, null=True, blank=True)
    username = models.CharField(max_length=128, null=True, blank=True)

    def is_private(self):
        return self.type == self.CHAT_TYPES[0][0]

    class Meta:
        abstract = True

    def __str__(self):
        return "{} ({})".format(self.title, self.telegram_id)


class AbstractTelegramUser(models.Model):
    """
    Represented a `user` type in Bot API:
        https://core.telegram.org/bots/api#user
    """
    telegram_id = models.CharField(max_length=128, unique=True)
    is_bot = models.BooleanField(default=False)
    first_name = models.CharField(max_length=128)
    last_name = models.CharField(max_length=128, null=True, blank=True)
    username = models.CharField(max_length=128, null=True, blank=True)

    def get_chat_state(self, chat: AbstractTelegramChat):
        state = AbstractTelegramState.objects.get_or_create(
            telegram_user__telegram_id=self.telegram_id,
            telegram_chat__telegram_id=chat.telegram_id
        )
        return state

    def get_private_chat_state(self):
        state = AbstractTelegramState.objects.get_or_create(
            telegram_user__telegram_id=self.telegram_id,
            telegram_chat__telegram_id=self.telegram_id
        )
        return state

    class Meta:
        abstract = True

    def __str__(self):
        return "{} {} (@{})".format(self.first_name, self.last_name, self.username)


class AbstractTelegramState(models.Model):
    memory = models.TextField(null=True, blank=True, verbose_name="Memory in JSON format")
    name = models.CharField(max_length=256, null=True, blank=True)

    class Meta:
        abstract = True

    def get_memory(self):
        """
        Gives a python object as the memory for this state.
        Use the set_memory method to set an object as memory. If invalid JSON used as memory, it will be cleared
        upon calling this method.
        """
        if self.memory in [None, '']:
            return {}
        try:
            return json.loads(self.memory)
        except ValueError:
            self.memory = ''
            self.save()
            return {}

    def set_memory(self, obj):
        """
        Sets a python object as memory for the state, in JSON format.
        If given object cannot be converted to JSON, function call will be ignored.
        :param obj: The memory object to be stored
        """
        try:
            self.memory = json.dumps(obj)
            self.save()
        except ValueError:
            pass

    def reset_memory(self):
        """
        Resets memory
        """
        self.set_memory({})

    def update_memory(self, obj):
        """
        Updates the memory in the exact way a Python dictionary is updated. New keys will be added and
        existing keys' value will be updated.
        :param obj: The dictionary to update based on
        """
        if type(obj) != dict:
            raise ValueError("Passed object should be type of dict")
        memory = self.get_memory()
        memory.update(obj)
        self.set_memory(memory)

    def set_name(self, name):
        self.name = name
        self.save()



from . import BasicType
from . import sticker


class StickerSet(BasicType):
    fields = {
        'name': str,
        'title': str,
        'is_animated': BasicType.bool_interpreter,
        'contains_masks': BasicType.bool_interpreter,
        'stickers': {
            'class': sticker.Sticker,
            'array': True
        }
    }

    def __init__(self, obj=None):
        super(StickerSet, self).__init__(obj)



from . import BasicType


class PreCheckoutQuery(BasicType):
    fields = {
        'id': str,
        'currency': str,
        'total_amount': int,
        'invoice_payload': str,
        'shipping_option_id': str,
    }

    def __init__(self, obj=None):
        super(PreCheckoutQuery, self).__init__(obj)

    def get_user(self):
        return getattr(self, 'from', None)

    def get_from(self):
        return self.get_user()


from . import user, orderinfo

PreCheckoutQuery.fields.update({
    'from': user.User,
    'order_info': orderinfo.OrderInfo
})



from . import BasicType


class LabeledPrice(BasicType):
    fields = {
        'label': str,
        'amount': int
    }

    def __init__(self, obj=None):
        super(LabeledPrice, self).__init__(obj)

    def get_label(self) -> str:
        return getattr(self, 'label', None)

    def get_amount(self) -> int:
        return getattr(self, 'amount', None)

    @classmethod
    def a(cls, label: str, amount: int):
        return super().a(**locals())


from . import BasicType


class MessageEntity(BasicType):
    fields = {
        'type': str,
        'offset': int,
        'length': int,
        'url': str,
        'language': str,
    }

    def __init__(self, obj=None):
        super(MessageEntity, self).__init__(obj)

    def get_type(self) -> str:
        return getattr(self, 'type', None)


from . import user
MessageEntity.fields.update({
    'user': user.User
})


from typing import Optional

from . import BasicType


class ChatPermissions(BasicType):
    fields = {
        'can_send_messages': BasicType.bool_interpreter,
        'can_send_media_messages': BasicType.bool_interpreter,
        'can_send_polls': BasicType.bool_interpreter,
        'can_send_other_messages': BasicType.bool_interpreter,
        'can_add_web_page_previews': BasicType.bool_interpreter,
        'can_change_info': BasicType.bool_interpreter,
        'can_invite_users': BasicType.bool_interpreter,
        'can_pin_messages': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(ChatPermissions, self).__init__(obj)

    @classmethod
    def a(cls, can_send_messages: Optional[bool] = None, can_send_media_messages: Optional[bool] = None,
          can_send_polls: Optional[bool] = None, can_send_other_messages: Optional[bool] = None,
          can_add_web_page_previews: Optional[bool] = None, can_change_info: Optional[bool] = None,
          can_invite_users: Optional[bool] = None, can_pin_messages: Optional[bool] = None):
        return super().a(**locals())




from django_tgbot.state_manager import state_types


class TransitionCondition:
    def __init__(self, from_states=None, message_types=None, update_types=None, exclude_message_types=None,
                 exclude_update_types=None):
        """
        Leave message_types / update_types for accepting all.
        :param from_states: the user's state in database
        :param message_types: accepted message_types
        :param update_types: accepted update_types
        """
        if message_types in ['all', '*', state_types.All]:
            message_types = None

        if update_types in ['all', '*', state_types.All]:
            update_types = None

        if from_states is None or from_states == state_types.Reset:
            from_states = ['', None]
        elif from_states in ['*', state_types.All]:
            from_states = ['*']

        args = locals()
        for var in args.keys():
            if var == 'self':
                continue

            result = args[var]
            if args[var] is None:
                result = []
            elif type(args[var]) == str:
                result = [args[var]]

            if type(result) != list:
                raise ValueError("Type of `{}` should be list".format(var))

            setattr(self, var, result)

    def matches(self, state_name, update_type, message_type=None):
        try:
            return (state_name in self.from_states or self.from_states == ['*']) and \
                   (message_type is None or message_type in self.message_types or self.message_types == []) and \
                   (message_type is None or message_type not in self.exclude_message_types) and \
                   (update_type in self.update_types or self.update_types == []) and \
                   (update_type not in self.exclude_update_types)
        except AttributeError:
            return False



from django_tgbot.exceptions import ProcessFailure
from django_tgbot.state_manager import state_types
from django_tgbot.state_manager.transition_condition import TransitionCondition
import inspect
from django_tgbot.state_manager.state_manager import StateManager


def processor(manager: StateManager, from_states=None, message_types=None, update_types=None,
              exclude_message_types=None, exclude_update_types=None, success=None, fail=None):
    def state_registrar(func):
        if func is None:
            raise ValueError("Passed processor is None.")
        all_args = inspect.getfullargspec(func)
        if not all([
            x in all_args[0] for x in ['bot', 'update', 'state']
        ]):
            raise ValueError("Passed processor does not have a valid signature.")

        def function_runner(bot, update, state, *args, **kwargs):
            current_state = state.name
            try:
                func(bot=bot, update=update, state=state, *args, **kwargs)
                if success == state_types.Reset:
                    state.name = ''
                    state.save()
                elif success == state_types.Keep:
                    state.name = current_state
                    state.save()
                elif success is not None:
                    state.name = success
                    state.save()
            except ProcessFailure:
                if fail == state_types.Reset:
                    state.name = ''
                    state.save()
                elif fail == state_types.Keep:
                    state.name = current_state
                    state.save()
                elif fail is not None:
                    state.name = fail
                    state.save()

        altered_message_types = message_types
        altered_update_types = update_types

        if altered_message_types is None:
            altered_message_types = manager.default_message_types
        if altered_update_types is None:
            altered_update_types = manager.default_update_types

        manager.register_state(TransitionCondition(
            from_states=from_states,
            message_types=altered_message_types,
            exclude_message_types=exclude_message_types,
            update_types=altered_update_types,
            exclude_update_types=exclude_update_types,
        ), processor=function_runner)

        return function_runner

    return state_registrar



from django_tgbot.bot import AbstractTelegramBot
from django_tgbot.state_manager.state_manager import StateManager
from django_tgbot.types.update import Update
from . import bot_token
from .models import TelegramUser, TelegramChat, TelegramState


class TelegramBot(AbstractTelegramBot):
    def __init__(self, token, state_manager):
        super(TelegramBot, self).__init__(token, state_manager)
    
    def get_db_user(self, telegram_id):
        return TelegramUser.objects.get_or_create(telegram_id=telegram_id)[0]

    def get_db_chat(self, telegram_id):
        return TelegramChat.objects.get_or_create(telegram_id=telegram_id)[0]

    def get_db_state(self, db_user, db_chat):
        return TelegramState.objects.get_or_create(telegram_user=db_user, telegram_chat=db_chat)[0]
    
    def pre_processing(self, update: Update, user, db_user, chat, db_chat, state):
        super(TelegramBot, self).pre_processing(update, user, db_user, chat, db_chat, state)

    def post_processing(self, update: Update, user, db_user, chat, db_chat, state):
        super(TelegramBot, self).post_processing(update, user, db_user, chat, db_chat, state)


state_manager = StateManager()
bot = TelegramBot(bot_token, state_manager)

from . import processors



from . import BasicType


class InlineQuery(BasicType):
    fields = {
        'id': str,
        'query': str,
        'offset': str
    }

    def __init__(self, obj=None):
        super(InlineQuery, self).__init__(obj)

    def get_user(self):  # -> user.User:
        return getattr(self, 'from', None)

    def get_id(self) -> str:
        return getattr(self, 'id', None)

    def get_query(self) -> str:
        return getattr(self, 'query', None)

    def get_offset(self) -> str:
        return getattr(self, 'offset', None)


from . import user, location

InlineQuery.fields.update({
    'from': user.User,
    'location': location.Location,
})


from django_tgbot.state_manager import update_types
from django_tgbot.types import chat
from . import BasicType
from . import message, inlinequery, choseninlineresult, callbackquery, shippingquery, precheckoutquery, poll, pollanswer


class Update(BasicType):
    fields = {
        'update_id': str,
        'message': message.Message,
        'edited_message': message.Message,
        'channel_post': message.Message,
        'edited_channel_post': message.Message,
        'inline_query': inlinequery.InlineQuery,
        'chosen_inline_result': choseninlineresult.ChosenInlineResult,
        'callback_query': callbackquery.CallbackQuery,
        'shipping_query': shippingquery.ShippingQuery,
        'pre_checkout_query': precheckoutquery.PreCheckoutQuery,
        'poll': poll.Poll,
        'poll_answer': pollanswer.PollAnswer
    }

    def __init__(self, obj=None):
        super(Update, self).__init__(obj)

    def type(self):
        field_checks = {
            'message': update_types.Message,
            'edited_message': update_types.EditedMessage,
            'channel_post': update_types.ChannelPost,
            'edited_channel_post': update_types.EditedChannelPost,
            'inline_query': update_types.InlineQuery,
            'chosen_inline_result': update_types.ChosenInlineResult,
            'callback_query': update_types.CallbackQuery,
            'shipping_query': update_types.ShippingQuery,
            'pre_checkout_query': update_types.PreCheckoutQuery,
            'poll': update_types.Poll,
            'poll_answer': update_types.PollAnswer
        }
        for key in field_checks.keys():
            if getattr(self, key, None) is not None:
                return field_checks[key]
        return None

    def get_message(self) -> message.Message or None:
        for key in ['message', 'edited_message', 'channel_post', 'edited_channel_post']:
            if getattr(self, key, None) is not None:
                return getattr(self, key, None)
        return None

    def get_chat(self) -> chat.Chat or None:
        if self.get_message() is not None:
            return self.get_message().get_chat()
        if self.is_callback_query():
            return self.get_callback_query().get_chat()
        return None

    def get_user(self):
        if self.get_message() is not None:
            return self.get_message().get_user()
        if self.is_inline_query():
            return self.get_inline_query().get_user()
        if self.is_chosen_inline_result():
            return self.get_chosen_inline_result().get_user()
        if self.is_callback_query():
            return self.get_callback_query().get_user()
        if self.is_shipping_query():
            return self.get_shipping_query().get_user()
        if self.is_pre_checkout_query():
            return self.get_pre_checkout_query().get_user()
        if self.is_poll_answer():
            return self.get_poll_answer().get_user()
        return None

    def get_callback_query(self) -> callbackquery.CallbackQuery:
        return getattr(self, 'callback_query', None)

    def get_inline_query(self) -> inlinequery.InlineQuery:
        return getattr(self, 'inline_query', None)

    def get_shipping_query(self) -> shippingquery.ShippingQuery:
        return getattr(self, 'shipping_query', None)

    def get_chosen_inline_result(self) -> choseninlineresult.ChosenInlineResult:
        return getattr(self, 'chosen_inline_result', None)

    def get_pre_checkout_query(self) -> precheckoutquery.PreCheckoutQuery:
        return getattr(self, 'pre_checkout_query', None)

    def get_poll_answer(self) -> pollanswer.PollAnswer:
        return getattr(self, 'poll_answer', None)

    def is_poll_answer(self):
        return self.get_poll_answer() is not None

    def is_pre_checkout_query(self):
        return self.get_pre_checkout_query() is not None

    def is_chosen_inline_result(self):
        return self.get_chosen_inline_result() is not None

    def is_inline_query(self):
        return self.get_inline_query() is not None

    def is_callback_query(self):
        return self.get_callback_query() is not None

    def is_shipping_query(self):
        return self.get_shipping_query() is not None



Text = 'text'
Audio = 'audio'
Document = 'document'
Animation = 'animation'
Game = 'game'
Photo = 'photo'
Sticker = 'sticker'
Video = 'video'
Voice = 'voice'
VideoNote = 'video_note'
Contact = 'contact'
Location = 'location'
Venue = 'venue'
Poll = 'poll'
NewChatMembers = 'new_chat_members'
LeftChatMember = 'left_chat_member'
NewChatTitle = 'new_chat_title'
NewChatPhoto = 'new_chat_photo'
DeleteChatPhoto = 'delete_chat_photo'
GroupChatCreated = 'group_chat_created'
SupergroupChatCreated = 'supergroup_chat_created'
ChannelChatCreated = 'channel_chat_created'
MigrateToChatId = 'migrate_to_chat_id'
MigrateFromChatId = 'migrate_from_chat_id'
PinnedMessage = 'pinned_message'
Invoice = 'invoice'
SuccessfulPayment = 'successful_payment'
PassportData = 'passport_data'



from typing import Optional

from . import BasicType
from . import keyboardbuttonpolltype


class KeyboardButton(BasicType):
    fields = {
        'text': str,
        'request_contact': BasicType.bool_interpreter,
        'request_location': BasicType.bool_interpreter,
        'request_poll': keyboardbuttonpolltype.KeyboardButtonPollType
    }

    def __init__(self, obj=None):
        super(KeyboardButton, self).__init__(obj)

    @classmethod
    def a(cls, text: str, request_contact: Optional[bool] = None, request_location: Optional[bool] = None,
          request_poll: Optional[keyboardbuttonpolltype.KeyboardButtonPollType] = None):
        return super().a(**locals())



from django_tgbot.decorators import processor
from django_tgbot.state_manager import message_types, update_types, state_types
from django_tgbot.types.update import Update
from .bot import state_manager
from .models import TelegramState
from .bot import TelegramBot


@processor(state_manager, from_states=state_types.All)
def hello_world(bot: TelegramBot, update: Update, state: TelegramState):
    bot.sendMessage(update.get_chat().get_id(), 'Hello!')



from django_tgbot.models import AbstractTelegramState
from django_tgbot.state_manager.transition_condition import TransitionCondition
from django_tgbot.types.update import Update


class StateManager:
    def __init__(self):
        self.handling_states = {}
        self.default_message_types = None
        self.default_update_types = None

    def register_state(self, state: TransitionCondition, processor):
        self.handling_states[state] = processor

    def get_processors(self, update: Update, state: AbstractTelegramState):
        """
        Searches through all of the processors and creates a list of those that handle the current state
        :param update: The received update
        :param state: The current state
        :return: a list of processors
        """
        message = update.get_message()
        message_type = message.type() if message is not None else None
        update_type = update.type()
        state_name = state.name
        processors = []
        for state in self.handling_states.keys():
            if state.matches(state_name=state_name, update_type=update_type, message_type=message_type):
                processors.append(self.handling_states[state])
        return processors

    def set_default_message_types(self, message_types):
        self.default_message_types = message_types

    def set_default_update_types(self, update_types):
        self.default_update_types = update_types




from typing import Optional

from . import BasicType


class ChosenInlineResult(BasicType):
    fields = {
        'result_id': str,
        'inline_message_id': str,
        'query': str
    }

    def __init__(self, obj=None):
        super(ChosenInlineResult, self).__init__(obj)

    def get_user(self):  # -> user.User:
        return getattr(self, 'from', None)

    def get_from(self):
        return self.get_user()

    def get_result_id(self) -> str:
        return getattr(self, 'result_id', None)

    def get_inline_message_id(self):  # -> Optional[str]
        return getattr(self, 'inline_message_id', None)

    def get_query(self) -> str:
        return getattr(self, 'query', None)

    def get_location(self):  # -> Optional[location.Location]
        return getattr(self, 'location', None)


# Placed here to avoid import cycles
from . import user, location

ChosenInlineResult.fields.update({
    'from': user.User,
    'location': location.Location,
})



from . import BasicType


class Voice(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'duration': int,
        'mime_type': str,
        'file_size': int
    }

    def __init__(self, obj=None):
        super(Voice, self).__init__(obj)




from django_tgbot.bot_api_user import BotAPIUser
from django_tgbot.types.user import User


def validate_token(bot_token):
    """
    If False, the second returned value is:
        0 if token is not valid
        1 if internet connection is not established
    If True, the second argument will be getMe result
    """
    api_user = BotAPIUser(bot_token)
    get_me_result = api_user.getMe()
    if type(get_me_result) != User:
        if 'no_connection' not in get_me_result.keys():
            return False, 0
        else:
            return False, 1
    return True, get_me_result


def prompt_token(command_line, prompt_message='Enter the bot token (retrieved from BotFather): '):
    """
    Returns either None or getMe result
    """
    bot_token = input(prompt_message)
    validation = validate_token(bot_token)
    while not validation[0]:
        if validation[1] == 0:
            bot_token = input("Bot token is not valid. Please enter again: ")
            validation = validate_token(bot_token)
        elif validation[1] == 1:
            command_line.stdout.write(
                command_line.style.ERROR(
                    "Connection failed. You need to be connected to the internet to run this command."
                )
            )
            return None, bot_token
    return validation[1], bot_token


def set_webhook(bot_token, url):
    api_user = BotAPIUser(bot_token)
    res = api_user.setWebhook(url)
    return res


def prompt_webhook(command_line, bot_token, bot_username):
    webhook_url = input("Enter the url to be set as webhook for @{}: ".format(bot_username))
    res = set_webhook(bot_token, webhook_url)
    if res['ok']:
        command_line.stdout.write(command_line.style.SUCCESS("Successfully set webhook."))
    else:
        command_line.stdout.write(command_line.style.WARNING("Couldn't set webhook:\n{}".format(res['description'])))


def prompt_project_url(command_line, bot_token, bot_username):
    project_url = input("Enter the url of this project to set the webhook (Press Enter to skip): ")
    while len(project_url) > 0 and project_url[-1] == '/':
        project_url = project_url[:-1]
    if project_url != '':
        confirmed = False
        while not confirmed:
            confirmed_text = input("Bot webhook will be set to {}/{}/update/. Do you confirm? (Y/N): ".format(
                project_url,
                bot_username
            ))
            confirmed = confirmed_text.lower() in ['yes', 'y']
            if not confirmed:
                project_url = input("Enter the correct url: ")
                while len(project_url) > 0 and project_url[-1] == '/':
                    project_url = project_url[:-1]

        res = set_webhook(bot_token, "{}/{}/update/".format(project_url, bot_username))
        if res['ok']:
            command_line.stdout.write(command_line.style.SUCCESS("Successfully set webhook."))
        else:
            command_line.stdout.write(command_line.style.WARNING("Couldn't set webhook:\n{}".format(res['description'])))



from typing import Optional

from django_tgbot.state_manager import message_types
from . import BasicType


class Message(BasicType):
    """
    Represents Telegram Message type:
        https://core.telegram.org/bots/api#message
    """
    fields = {
        'message_id': str,
        'date': str,
        'forward_from_message_id': str,
        'forward_signature': str,
        'forward_sender_name': str,
        'forward_date': str,
        'edit_date': str,
        'media_group_id': str,
        'author_signature': str,
        'text': str,
        'caption': str,
        'new_chat_title': str,
        'delete_chat_photo': BasicType.bool_interpreter,
        'group_chat_created': BasicType.bool_interpreter,
        'supergroup_chat_created': BasicType.bool_interpreter,
        'channel_chat_created': BasicType.bool_interpreter,
        'migrate_to_chat_id': str,
        'migrate_from_chat_id': str,
        'connected_website': str,
    }

    def __init__(self, obj=None):
        super(Message, self).__init__(obj)

    def type(self):
        field_checks = {
            'text': message_types.Text,
            'audio': message_types.Audio,
            'document': message_types.Document,
            'animation': message_types.Animation,
            'game': message_types.Game,
            'photo': message_types.Photo,
            'sticker': message_types.Sticker,
            'video': message_types.Video,
            'voice': message_types.Voice,
            'video_note': message_types.VideoNote,
            'contact': message_types.Contact,
            'location': message_types.Location,
            'venue': message_types.Venue,
            'poll': message_types.Poll,
            'new_chat_members': message_types.NewChatMembers,
            'left_chat_member': message_types.LeftChatMember,
            'new_chat_title': message_types.NewChatTitle,
            'new_chat_photo': message_types.NewChatPhoto,
            'delete_chat_photo': message_types.DeleteChatPhoto,
            'group_chat_created': message_types.GroupChatCreated,
            'supergroup_chat_created': message_types.SupergroupChatCreated,
            'channel_chat_created': message_types.ChannelChatCreated,
            'migrate_to_chat_id': message_types.MigrateToChatId,
            'migrate_from_chat_id': message_types.MigrateFromChatId,
            'pinned_message': message_types.PinnedMessage,
            'invoice': message_types.Invoice,
            'successful_payment': message_types.SuccessfulPayment,
            'passport_data': message_types.PassportData
        }

        for key in field_checks.keys():
            if getattr(self, key, None) is not None:
                return field_checks[key]
        return None

    def get_message_id(self) -> str:
        return getattr(self, 'message_id', None)

    def get_chat(self):
        return getattr(self, 'chat', None)

    def get_user(self):
        return getattr(self, 'from', None)

    def get_from(self):
        return self.get_user()

    def get_reply_to_message(self):
        return getattr(self, 'reply_to_message', None)

    def get_text(self) -> Optional[str]:
        return getattr(self, 'text', None)

    def get_caption(self) -> Optional[str]:
        return getattr(self, 'caption', None)

    def get_reply_markup(self):
        return getattr(self, 'reply_markup', None)


# Placed here to avoid import cycles
from . import user, chat, messageentity, audio, document, animation, game, photosize, \
    inlinekeyboardmarkup, passportdata, successfulpayment, invoice, poll, \
    venue, location, contact, videonote, voice, video, sticker


Message.fields.update({
    'reply_to_message': Message,
    'pinned_message': Message,
    'from': user.User,
    'chat': chat.Chat,
    'forward_from': user.User,
    'forward_from_chat': chat.Chat,
    'entities': {
        'class': messageentity.MessageEntity,
        'array': True
    },
    'caption_entities': {
        'class': messageentity.MessageEntity,
        'array': True
    },
    'audio': audio.Audio,
    'document': document.Document,
    'animation': animation.Animation,
    'game': game.Game,
    'photo': {
        'class': photosize.PhotoSize,
        'array': True
    },
    'sticker': sticker.Sticker,
    'video': video.Video,
    'voice': voice.Voice,
    'video_note': videonote.VideoNote,
    'contact': contact.Contact,
    'location': location.Location,
    'venue': venue.Venue,
    'poll': poll.Poll,
    'new_chat_members': {
        'class': user.User,
        'array': True
    },
    'left_chat_member': user.User,
    'new_chat_photo': {
        'class': photosize.PhotoSize,
        'array': True
    },
    'invoice': invoice.Invoice,
    'successful_payment': successfulpayment.SuccessfulPayment,
    'passport_data': passportdata.PassportData,
    'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup
})



from . import BasicType


class ShippingQuery(BasicType):
    fields = {
        'id': str,
        'invoice_payload': str
    }

    def __init__(self, obj=None):
        super(ShippingQuery, self).__init__(obj)

    def get_user(self):
        return getattr(self, 'from', None)

    def get_from(self):
        return self.get_user()


from . import user, shippingaddress

ShippingQuery.fields.update({
    'from': user.User,
    'shipping_address': shippingaddress.ShippingAddress
})


from . import BasicType


class GameHighScore(BasicType):
    fields = {
        'position': int,
        'score': int
    }

    def __init__(self, obj=None):
        super(GameHighScore, self).__init__(obj)

    def get_user(self):  # -> user.User:
        return getattr(self, 'user', None)

    def get_score(self) -> int:
        return getattr(self, 'score', None)


from . import user

GameHighScore.fields.update({
    'user': user.User,
})


from typing import Optional

from . import BasicType


class KeyboardButtonPollType(BasicType):
    fields = {
        'type': str,
    }

    def __init__(self, obj=None):
        super(KeyboardButtonPollType, self).__init__(obj)

    @classmethod
    def a(cls, type: Optional[str] = None):
        return super().a(**locals())



from . import BasicType
from . import photosize


class VideoNote(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'length': int,
        'duration': int,
        'file_size': int,
        'thumb': photosize.PhotoSize
    }

    def __init__(self, obj=None):
        super(VideoNote, self).__init__(obj)



from . import BasicType


class PollOption(BasicType):
    fields = {
        'text': str,
        'voter_count': int
    }

    def __init__(self, obj=None):
        super(PollOption, self).__init__(obj)



from . import BasicType


class Game(BasicType):
    fields = {
        'title': str,
        'description': str,
        'text': str
    }

    def __init__(self, obj=None):
        super(Game, self).__init__(obj)

    def get_title(self) -> str:
        return getattr(self, 'title', None)


# Placed here to avoid import cycles
from . import photosize, messageentity, animation

Game.fields.update({
    'photo': {
        'class': photosize.PhotoSize,
        'array': True
    },
    'text_entities': {
        'class': messageentity.MessageEntity,
        'array': True
    },
    'animation': animation.Animation
})




from . import BasicType


class UserProfilePhotos(BasicType):
    fields = {
        'total_count': int,
    }

    def __init__(self, obj=None):
        super(UserProfilePhotos, self).__init__(obj)


from . import photosize

UserProfilePhotos.fields.update({
    'photos': {
        'class': photosize.PhotoSize,
        'array_of_array': True
    }
})


from typing import List

from . import BasicType
from . import inlinekeyboardbutton


class InlineKeyboardMarkup(BasicType):
    fields = {
        'inline_keyboard': {
            'class': inlinekeyboardbutton.InlineKeyboardButton,
            'array_of_array': True
        }
    }

    def __init__(self, obj=None):
        super(InlineKeyboardMarkup, self).__init__(obj)

    @classmethod
    def a(cls, inline_keyboard: List[List[inlinekeyboardbutton.InlineKeyboardButton]]):
        return super().a(**locals())



from typing import Optional

from . import BasicType


class ReplyKeyboardRemove(BasicType):
    fields = {
        'remove_keyboard': BasicType.bool_interpreter,
        'selective': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(ReplyKeyboardRemove, self).__init__(obj)

    @classmethod
    def a(cls, remove_keyboard: bool, selective: Optional[bool] = None):
        return super().a(**locals())



import errno
import os
import shutil

from django.conf import settings
from django.core.management.base import BaseCommand

from django_tgbot.management import helpers


class Command(BaseCommand):
    help = 'Creates a new Telegram bot'

    def handle(self, *args, **options):

        get_me_result, bot_token = helpers.prompt_token(self)
        bot_name = str(get_me_result.get_first_name())
        bot_username = str(get_me_result.get_username()).lower()

        self.stdout.write('Setting up @{} ...'.format(bot_username))

        helpers.prompt_project_url(self, bot_token, bot_username)

        dst = os.path.join(settings.BASE_DIR, bot_username)
        if os.path.isdir(dst):
            self.stdout.write(self.style.ERROR('Directory `{}` already exists.'.format(dst)))
            return

        src = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'bot_template')
        try:
            shutil.copytree(src, dst)
        except OSError as exc:
            if exc.errno == errno.ENOTDIR:
                shutil.copy(src, dst)
            else:
                self.stdout.write(self.style.ERROR("Couldn't copy bot files."))
                return

        with open(os.path.join(dst, '__init__.py'), 'w') as f:
            f.write("# Do not remove these 2 lines:\nbot_token = '{}'\napp_name='{}'\n".format(bot_token, bot_username))

        self.stdout.write(self.style.SUCCESS('Successfully created bot {}(@{}).'.format(bot_name, bot_username)))

        self.stdout.write("Next steps:")
        self.stdout.write("\t1. Add '{}' to INSTALLED_APPS in project settings".format(bot_username))
        self.stdout.write("\t2. Add `from {} import urls as {}_urls` to project urls file".format(bot_username, bot_username))
        self.stdout.write("\t3. Add `path('{}/', include({}_urls))` to project urls' urlpatterns".format(bot_username, bot_username))
        self.stdout.write("\t4. `python manage.py migrate`")
        self.stdout.write("Enjoy!")



from typing import Optional

from . import BasicType


class Contact(BasicType):
    fields = {
        'phone_number': str,
        'first_name': str,
        'last_name': str,
        'user_id': str,
        'vcard': str
    }

    def __init__(self, obj=None):
        super(Contact, self).__init__(obj)

    def get_phone_number(self) -> str:
        return getattr(self, 'phone_number', None)

    def get_first_name(self) -> str:
        return getattr(self, 'first_name', None)

    def get_last_name(self) -> Optional[str]:
        return getattr(self, 'last_name', None)

    @classmethod
    def a(cls, phone_number: str, first_name: str, last_name: Optional[str] = None, user_id: Optional[str] = None,
          vcard: Optional[str] = None):
        return super().a(**locals())


Message = 'message'
EditedMessage = 'edited_message'
ChannelPost = 'channel_post'
EditedChannelPost = 'edited_channel_post'
InlineQuery = 'inline_query'
ChosenInlineResult = 'chosen_inline_result'
CallbackQuery = 'callback_query'
ShippingQuery = 'shipping_query'
PreCheckoutQuery = 'pre_checkout_query'
Poll = 'poll'
PollAnswer = 'poll_answer'



from typing import Optional

from . import BasicType


class LoginUrl(BasicType):
    fields = {
        'url': str,
        'forward_text': str,
        'bot_username': str,
        'request_write_access': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(LoginUrl, self).__init__(obj)

    @classmethod
    def a(cls, url: str, forward_text: Optional[str] = None, bot_username: Optional[str] = None, request_write_access: Optional[bool] = None):
        return super().a(**locals())


from django.db import models
from django.db.models import CASCADE

from django_tgbot.models import AbstractTelegramUser, AbstractTelegramChat, AbstractTelegramState


class TelegramUser(AbstractTelegramUser):
    pass


class TelegramChat(AbstractTelegramChat):
    pass


class TelegramState(AbstractTelegramState):
    telegram_user = models.ForeignKey(TelegramUser, related_name='telegram_states', on_delete=CASCADE)
    telegram_chat = models.ForeignKey(TelegramChat, related_name='telegram_states', on_delete=CASCADE)

    class Meta:
        unique_together = ('telegram_user', 'telegram_chat')




from . import BasicType


class ChatMember(BasicType):
    fields = {
        'status': str,
        'custom_title': str,
        'until_date': str,
        'can_be_edited': BasicType.bool_interpreter,
        'can_post_messages': BasicType.bool_interpreter,
        'can_edit_messages': BasicType.bool_interpreter,
        'can_delete_messages': BasicType.bool_interpreter,
        'can_restrict_members': BasicType.bool_interpreter,
        'can_promote_members': BasicType.bool_interpreter,
        'can_change_info': BasicType.bool_interpreter,
        'can_invite_users': BasicType.bool_interpreter,
        'can_pin_messages': BasicType.bool_interpreter,
        'is_member': BasicType.bool_interpreter,
        'can_send_messages': BasicType.bool_interpreter,
        'can_send_media_messages': BasicType.bool_interpreter,
        'can_send_polls': BasicType.bool_interpreter,
        'can_send_other_messages': BasicType.bool_interpreter,
        'can_add_web_page_previews': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(ChatMember, self).__init__(obj)

    def get_user(self):
        return getattr(self, 'user', None)

    def get_status(self) -> str:
        return getattr(self, 'status', None)


# Placed here to avoid import cycles
from . import user

ChatMember.fields.update({
    'user': user.User
})



from typing import Optional

from django_tgbot.types import chat
from . import BasicType


class CallbackQuery(BasicType):
    fields = {
        'id': str,
        'inline_message_id': str,
        'chat_instance': str,
        'data': str,
        'game_short_name': str
    }

    def __init__(self, obj=None):
        super(CallbackQuery, self).__init__(obj)

    def get_id(self) -> str:
        return getattr(self, 'id', None)

    def get_user(self):
        return getattr(self, 'from', None)

    def get_from(self):
        return self.get_user()

    def get_message(self):  # -> Optional[message.Message]:
        return getattr(self, 'message', None)

    def get_chat(self) -> Optional[chat.Chat]:
        if self.get_message() is None:
            return None
        return self.get_message().get_chat()

    def get_inline_message_id(self) -> Optional[str]:
        return getattr(self, 'inline_message_id', None)

    def get_data(self) -> Optional[str]:
        return getattr(self, 'data', None)

# Placed here to avoid import cycles

from . import user, message

CallbackQuery.fields.update({
    'from': user.User,
    'message': message.Message
})



from . import BasicType
from . import encryptedpassportelement, encryptedcredentials


class PassportData(BasicType):
    fields = {
        'data': {
            'class': encryptedpassportelement.EncryptedPassportElement,
            'array': True
        },
        'credentials': encryptedcredentials.EncryptedCredentials
    }

    def __init__(self, obj=None):
        super(PassportData, self).__init__(obj)



from . import BasicType


class Location(BasicType):
    fields = {
        'longitude': str,
        'latitude': str
    }

    def __init__(self, obj=None):
        super(Location, self).__init__(obj)

    @classmethod
    def a(cls, latitude: str, longitude: str):
        return super().a(**locals())



from . import BasicType
from . import orderinfo


class SuccessfulPayment(BasicType):
    fields = {
        'currency': str,
        'total_amount': int,
        'invoice_payload': str,
        'shipping_option_id': str,
        'order_info': orderinfo.OrderInfo,
        'telegram_payment_charge_id': str,
        'provider_payment_charge_id': str,
    }

    def __init__(self, obj=None):
        super(SuccessfulPayment, self).__init__(obj)


from . import BasicType


class User(BasicType):
    fields = {
        'id': str,
        'is_bot': BasicType.bool_interpreter,
        'first_name': str,
        'last_name': str,
        'username': str,
        'language_code': str,
        'can_join_groups': BasicType.bool_interpreter,
        'can_read_all_group_messages': BasicType.bool_interpreter,
        'supports_inline_queries': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(User, self).__init__(obj)

    def get_id(self):
        return getattr(self, 'id', None)

    def get_username(self):
        return getattr(self, 'username', None)

    def get_first_name(self):
        return getattr(self, 'first_name', None)

    def get_last_name(self):
        return getattr(self, 'last_name', None)



from . import BasicType


class ChatPhoto(BasicType):
    fields = {
        'small_file_id': str,
        'small_file_unique_id': str,
        'big_file_id': str,
        'big_file_unique_id': str
    }

    def __init__(self, obj=None):
        super(ChatPhoto, self).__init__(obj)


from . import BasicType
from . import polloption


class Poll(BasicType):
    fields = {
        'id': str,
        'question': str,
        'options': {
            'class': polloption.PollOption,
            'array': True
        },
        'total_voter_count': int,
        'is_closed': BasicType.bool_interpreter,
        'is_anonymous': BasicType.bool_interpreter,
        'type': str,
        'allows_multiple_answers': BasicType.bool_interpreter,
        'correct_option_id': int
    }

    def __init__(self, obj=None):
        super(Poll, self).__init__(obj)




from . import BasicType


class ShippingAddress(BasicType):
    fields = {
        'country_code': str,
        'state': str,
        'city': str,
        'street_line1': str,
        'street_line2': str,
        'post_code': str
    }

    def __init__(self, obj=None):
        super(ShippingAddress, self).__init__(obj)




from . import BasicType


class EncryptedCredentials(BasicType):
    fields = {
        'data': str,
        'hash': str,
        'secret': str
    }

    def __init__(self, obj=None):
        super(EncryptedCredentials, self).__init__(obj)

    def get_data(self) -> str:
        return getattr(self, 'data', None)

    def get_hash(self) -> str:
        return getattr(self, 'hash', None)

    def get_secret(self) -> str:
        return getattr(self, 'secret', None)


from typing import Optional

from . import BasicType


class InputMessageContent(BasicType):
    def __init__(self, obj=None):
        super(InputMessageContent, self).__init__(obj)


class InputTextMessageContent(InputMessageContent):
    fields = {
        'message_text': str,
        'parse_mode': str,
        'disable_web_page_preview': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(InputTextMessageContent, self).__init__(obj)

    @classmethod
    def a(cls, message_text: str,
          parse_mode: Optional[str] = None, disable_web_page_preview: Optional[bool] = None):
        return super().a(**locals())


class InputLocationMessageContent(InputMessageContent):
    fields = {
        'latitude': str,
        'longitude': str,
        'live_period': int
    }

    def __init__(self, obj=None):
        super(InputLocationMessageContent, self).__init__(obj)

    @classmethod
    def a(cls, latitude: str, longitude: str, live_period: Optional[int] = None):
        return super().a(**locals())


class InputVenueMessageContent(InputMessageContent):
    fields = {
        'latitude': str,
        'longitude': str,
        'title': str,
        'address': str,
        'foursquare_id': str,
        'foursquare_type': str
    }

    def __init__(self, obj=None):
        super(InputVenueMessageContent, self).__init__(obj)

    @classmethod
    def a(cls, latitude: str, longitude: str, title: str, address: str, foursquare_id: Optional[str] = None,
          foursquare_type: Optional[str] = None):
        return super().a(**locals())


class InputContactMessageContent(InputMessageContent):
    fields = {
        'phone_number': str,
        'first_name': str,
        'last_name': str,
        'vcard': str
    }

    def __init__(self, obj=None):
        super(InputContactMessageContent, self).__init__(obj)

    @classmethod
    def a(cls, phone_number: str, first_name: str, last_name: Optional[str] = None, vcard: Optional[str] = None):
        return super().a(**locals())





from typing import Optional

from . import BasicType
from . import loginurl


class InlineKeyboardButton(BasicType):
    fields = {
        'text': str,
        'url': str,
        'login_url': loginurl.LoginUrl,
        'callback_data': str,
        'switch_inline_query': str,
        'switch_inline_query_current_chat': str,
        'callback_game': str,
        'pay': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(InlineKeyboardButton, self).__init__(obj)

    @classmethod
    def a(cls, text: str, url: Optional[str] = None, login_url: Optional[loginurl.LoginUrl] = None,
          callback_data: Optional[str] = None, switch_inline_query: Optional[str] = None,
          switch_inline_query_current_chat: Optional[str] = None, callback_game: Optional[str] = None,
          pay: Optional[bool] = None):
        return super().a(**locals())



import os

from django.conf import settings
from django.core.management.base import BaseCommand

from django_tgbot.management import helpers

import importlib.util


class Command(BaseCommand):
    help = 'Updates the webhook for an existing tgbot'

    def handle(self, *args, **options):
        bot_username = input('Enter the username of the bot (without @): ').lower()
        dst = os.path.join(settings.BASE_DIR, bot_username)
        init_file = os.path.join(dst, '__init__.py')

        if not os.path.isfile(init_file):
            self.stdout.write(
                self.style.ERROR('No such bot found. Make sure you have created your bot with command `createtgbot`.')
            )
            return

        spec = importlib.util.spec_from_file_location("__init__", init_file)
        init_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(init_module)

        bot_token = init_module.bot_token

        again = True
        while again:
            choice = input("Please choose from the options:\n"
                           " 1. I want to change the project URL and use default webhooks\n"
                           " 2. I want to set a custom webhook URL for this bot\n")
            try:
                if int(choice) == 1:
                    helpers.prompt_project_url(self, bot_token, bot_username)
                    again = False
                elif int(choice) == 2:
                    helpers.prompt_webhook(self, bot_token, bot_username)
                    again = False
                else:
                    self.stdout.write('Did not recognize choice. Try again: ')
                    again = True
            except ValueError:
                self.stdout.write('Respond only with the number of your selected option: ')
                again = True



from typing import Optional, List

from . import BasicType
from . import passportfile


class EncryptedPassportElement(BasicType):
    fields = {
        'type': str,
        'data': str,
        'phone_number': str,
        'email': str,
        'files': {
            'class': passportfile.PassportFile,
            'array': True
        },
        'front_side': passportfile.PassportFile,
        'reverse_side': passportfile.PassportFile,
        'selfie': passportfile.PassportFile,
        'translation': {
            'class': passportfile.PassportFile,
            'array': True
        },
        'hash': str
    }

    def __init__(self, obj=None):
        super(EncryptedPassportElement, self).__init__(obj)

    def get_type(self) -> str:
        return getattr(self, 'type', None)

    def get_data(self) -> Optional[str]:
        return getattr(self, 'data', None)

    def get_files(self) -> Optional[List[passportfile.PassportFile]]:
        return getattr(self, 'files', None)



from . import BasicType


class Invoice(BasicType):
    fields = {
        'title': str,
        'description': str,
        'start_parameter': str,
        'currency': str,
        'total_amount': int
    }

    def __init__(self, obj=None):
        super(Invoice, self).__init__(obj)

    def get_title(self) -> str:
        return getattr(self, 'title', None)

    def get_start_parameter(self) -> str:
        return getattr(self, 'start_parameter', None)

    def get_currency(self) -> str:
        return getattr(self, 'currency', None)

    def total_amount(self) -> int:
        return getattr(self, 'total_amount', None)

    @classmethod
    def a(cls, title: str, description: str, start_parameter: str, currency: str, total_amount: int):
        return super().a(**locals())



from . import BasicType


class ResponseParameters(BasicType):
    fields = {
        'migrate_to_chat_id': str,
        'retry_after': int
    }

    def __init__(self, obj=None):
        super(ResponseParameters, self).__init__(obj)




from . import BasicType


class File(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'file_path': str,
        'file_size': int
    }

    def __init__(self, obj=None):
        super(File, self).__init__(obj)

    def get_file_id(self) -> str:
        return getattr(self, 'file_id', None)

    def get_file_unique_id(self) -> str:
        return getattr(self, 'file_unique_id', None)



from django_tgbot.bot_api_user import BotAPIUser
from django_tgbot.types.update import Update


class AbstractTelegramBot(BotAPIUser):
    def __init__(self, token, state_manager):
        super(AbstractTelegramBot, self).__init__(token)
        self.state_manager = state_manager

    def handle_update(self, update: Update):
        user = update.get_user()
        chat = update.get_chat()

        if user is not None:
            db_user = self.get_db_user(user.get_id())
        else:
            db_user = None

        if chat is not None:
            db_chat = self.get_db_chat(chat.get_id())
        else:
            db_chat = None

        db_state = self.get_db_state(db_user, db_chat)

        self.pre_processing(
            update,
            chat=chat,
            db_chat=db_chat,
            user=user,
            db_user=db_user,
            state=db_state
        )

        processors = self.state_manager.get_processors(update, db_state)

        for processor in processors:
            processor(self, update, db_state)

        self.post_processing(
            update,
            chat=chat,
            db_chat=db_chat,
            user=user,
            db_user=db_user,
            state=db_state
        )

    def pre_processing(self, update: Update, user, db_user, chat, db_chat, state):
        if db_user is not None:
            db_user.first_name = user.get_first_name()
            db_user.last_name = user.get_last_name()
            db_user.username = user.get_username()
            db_user.save()

        if db_chat is not None:
            db_chat.type = chat.get_type()
            db_chat.username = chat.get_username()
            db_chat.title = chat.get_title()
            db_chat.save()

    def post_processing(self, update: Update, user, db_user, chat, db_chat, state):
        pass

    def get_db_user(self, telegram_id):
        """
        Should be implemented - Creates or retrieves the user object from database
        :param telegram_id: The telegram user's id
        :return: User object from database
        """
        pass

    def get_db_chat(self, telegram_id):
        """
        Should be implemented - Creates or retrieves the chat object from database
        :param telegram_id: The telegram chat's id
        :return: Chat object from database
        """
        pass

    def get_db_state(self, db_user, db_chat):
        """
        Should be implemented - Creates or retrieves a state object in the database for this user and chat
        :param db_user: The user creating this state for
        :param db_chat: The related chat
        :return: a state object from database
        """
        pass



from . import BasicType


class PollAnswer(BasicType):
    fields = {
        'poll_id': str,
        'option_ids': {
            'class': int,
            'array': True
        },
    }

    def __init__(self, obj=None):
        super(PollAnswer, self).__init__(obj)

    def get_user(self):
        return getattr(self, 'user', None)


from . import user

PollAnswer.fields.update({
    'user': user.User
})


from . import BasicType


class Venue(BasicType):
    fields = {
        'title': str,
        'address': str,
        'foursquare_id': str,
        'foursquare_type': str,
    }

    def __init__(self, obj=None):
        super(Venue, self).__init__(obj)


from . import location

Venue.fields.update({
    'location': location.Location
})


from django.conf import settings
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from .bot import bot
from django_tgbot.types.update import Update

import logging


@csrf_exempt
def handle_bot_request(request):
    update = Update(request.body.decode("utf-8"))
    """
    All of the processing will happen in this part. It is wrapped in try-except block
    to make sure the returned HTTP status is 200. Otherwise, if your processors raise Exceptions
    causing this function to raise Exception and not return 200 status code, Telegram will stop
    sending updates to your webhook after a few tries. Instead, take the caught exception and handle it
    or log it to use for debugging later.
    """
    try:
        bot.handle_update(update)
    except Exception as e:
        if settings.DEBUG:
            raise e
        else:
            logging.exception(e)
    return HttpResponse("OK")



from typing import List, Optional

from . import BasicType
from . import keyboardbutton


class ReplyKeyboardMarkup(BasicType):
    fields = {
        'keyboard': {
            'class': keyboardbutton.KeyboardButton,
            'array_of_array': True
        },
        'resize_keyboard': BasicType.bool_interpreter,
        'one_time_keyboard': BasicType.bool_interpreter,
        'selective': BasicType.bool_interpreter
    }

    def __init__(self, obj=None):
        super(ReplyKeyboardMarkup, self).__init__(obj)

    @classmethod
    def a(cls, keyboard: List[List[keyboardbutton.KeyboardButton]], resize_keyboard: Optional[bool] = None,
          one_time_keyboard: Optional[bool] = None, selective: Optional[bool] = None):
        return super().a(**locals())




from . import BasicType


class Animation(BasicType):
    fields = {
        'file_id': str,
        'file_unique_id': str,
        'width': int,
        'height': int,
        'duration': int,
        'file_name': str,
        'mime_type': str,
        'file_size': int,
    }

    def __init__(self, obj=None):
        super(Animation, self).__init__(obj)

    def get_file_id(self):
        return getattr(self, 'file_id', None)


# Placed here to avoid import cycles

from . import photosize

Animation.fields.update({
    'thumb': photosize.PhotoSize
})



from . import BasicType


class MaskPosition(BasicType):
    fields = {
        'point': str,
        'x_shift': str,
        'y_shift': str,
        'scale': str
    }

    def __init__(self, obj=None):
        super(MaskPosition, self).__init__(obj)

    @classmethod
    def a(cls, point: str, x_shift: str, y_shift: str, scale: str):
        return super().a(**locals())



from typing import Union

import requests
import time
import json
import inspect

from django_tgbot.types.chat import Chat
from django_tgbot.types.chatmember import ChatMember
from django_tgbot.types.file import File
from django_tgbot.types.forcereply import ForceReply
from django_tgbot.types.inlinekeyboardmarkup import InlineKeyboardMarkup
from django_tgbot.types.message import Message
from django_tgbot.types.poll import Poll
from django_tgbot.types.replykeyboardmarkup import ReplyKeyboardMarkup
from django_tgbot.types.replykeyboardremove import ReplyKeyboardRemove
from django_tgbot.types.stickerset import StickerSet
from django_tgbot.types.user import User
from django_tgbot.types.userprofilephotos import UserProfilePhotos


def create_params_from_args(args=None, exclude=None):
    """
    The args should usually be `locals()` and exclude should be the args you want to exclude
    'self' is automatically added to exclude list.
    :return: a dictionary made from the arguments
    """
    if args is None:
        args = {}
    if exclude is None:
        exclude = []
    if 'self' not in exclude:
        exclude.append('self')
    result = {}
    for arg in args.keys():
        if arg in exclude or args[arg] is None:
            continue
        if hasattr(args[arg], 'to_dict'):
            result[arg] = args[arg].to_dict()
        else:
            result[arg] = args[arg]

        if arg == 'reply_markup' and type(result[arg]) != str:
            result[arg] = json.dumps(result[arg])

    return result


class BotAPIUser:
    MAX_TRIES = 5

    PARSE_MODE_MARKDOWN = 'Markdown'
    PARSE_MODE_HTML = 'HTML'

    POLL_TYPE_QUIZ = 'quiz'
    POLL_TYPE_REGULAR = 'regular'

    CHAT_ACTION_TYPING = 'typing'
    CHAT_ACTION_PHOTO = 'upload_photo'
    CHAT_ACTION_VIDEO_RECORD = 'record_video'
    CHAT_ACTION_VIDEO_UPLOAD = 'upload_video'
    CHAT_ACTION_AUDIO_RECORD = 'record_audio'
    CHAT_ACTION_AUDIO_UPLOAD = 'upload_audio'
    CHAT_ACTION_DOCUMENT = 'upload_document'
    CHAT_ACTION_LOCATION = 'find_location'
    CHAT_ACTION_VIDEO_NOTE_RECORD = 'record_video_note'
    CHAT_ACTION_VIDEO_NOTE_UPLOAD = 'upload_video_note'

    def __init__(self, token):
        self.token = ''
        self.api_url = ''
        self.set_token(token)

    def set_token(self, token):
        self.token = token
        self.api_url = 'https://api.telegram.org/bot{}'.format(self.token)

    def send_request(self, method, data=None, files=None):
        if data is None:
            data = {}
        url = '{}/{}'.format(self.api_url, method)
        r = None
        sleep_time = 1
        for i in range(self.MAX_TRIES):
            try:
                if files is None:
                    r = requests.post(url, data)
                else:
                    r = requests.post(url, data=data, files=files)
            except requests.RequestException:
                time.sleep(sleep_time)
                sleep_time += 0.2
                continue
            else:
                break

        if r is not None:
            python_result = json.loads(r.text)
            if 'ok' not in python_result:
                python_result['ok'] = bool(r)
            return python_result
        return {'ok': False, 'description': 'Max tries exceeded.', 'no_connection': True}

    def request_and_result(self, data, result_type, files=None):
        """
        Should be called by a method with exact same name as the API method
        """
        res = self.send_request(inspect.stack()[1].function, data=data, files=files)
        if res['ok']:
            return result_type(res['result'])
        else:
            return res

    def getMe(self) -> User:
        return self.request_and_result(create_params_from_args(), User)

    def getUpdates(self):
        pass   # TODO

    def setWebhook(self, url):
        return self.send_request('setWebhook', {'url': url})

    def sendMessage(self, chat_id, text, parse_mode=None, disable_web_page_preview=None, disable_notification=None,
                    reply_to_message_id=None,
                    reply_markup: Union[None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:
        return self.request_and_result(create_params_from_args(locals()), Message)

    def forwardMessage(self, chat_id, from_chat_id, message_id, disable_notification=None):
        return self.request_and_result(create_params_from_args(locals()), Message)

    def sendPhoto(self, chat_id, photo, upload=False, caption=None, parse_mode=None, disable_notification=None,
                  reply_to_message_id=None,
                  reply_markup: Union[None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:
        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'photo']),
                Message,
                files={'photo': photo}
            )

    def sendAudio(self, chat_id, audio, upload=False, caption=None, parse_mode=None, duration=None, performer=None, title=None,
                  thumb=None, disable_notification=None, reply_to_message_id=None,
                  reply_markup: Union[
                      None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'audio']),
                Message,
                files={'audio': audio}
            )

    def sendDocument(self, chat_id, document, upload=False, thumb=None, caption=None, parse_mode=None, disable_notification=None,
                     reply_to_message_id=None,
                     reply_markup: Union[None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply]=None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'document']),
                Message,
                files={'document': document}
            )

    def sendVideo(self, chat_id, video, upload=False, duration=None, width=None, height=None, thumb=None, caption=None,
                  parse_mode=None, supports_streaming=None, disable_notification=None, reply_to_message_id=None,
                  reply_markup: Union[
                      None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'video']),
                Message,
                files={'video': video}
            )

    def sendAnimation(self, chat_id, animation, upload=False, duration=None, width=None, height=None, thumb=None, caption=None,
                      parse_mode=None, disable_notification=None, reply_to_message_id=None,
                      reply_markup: Union[
                          None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'animation']),
                Message,
                files={'animation': animation}
            )

    def sendVoice(self, chat_id, voice, upload=False, caption=None, parse_mode=None, duration=None, disable_notification=None,
                  reply_to_message_id=None,
                  reply_markup: Union[
                      None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'voice']),
                Message,
                files={'voice': voice}
            )

    def sendVideoNote(self, chat_id, video_note, upload=False, duration=None, length=None, thumb=None, disable_notification=None,
                      reply_to_message_id=None,
                      reply_markup: Union[
                          None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'video_note']),
                Message,
                files={'video_note': video_note}
            )

    def sendMediaGroup(self, chat_id, media, upload=False, disable_notification=None, reply_to_message_id=None):
        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'media']),
                Message,
                files={'media': media}
            )

    def sendLocation(self, chat_id, latitude, longitude, live_period=None, disable_notification=None,
                     reply_to_message_id=None,
                     reply_markup: Union[
                         None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def editMessageLiveLocation(self, chat_id, latitude, longitude, message_id=None, inline_message_id=None,
                                reply_markup: Union[
                                    None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def stopMessageLiveLocation(self, chat_id, message_id=None, inline_message_id=None,
                                reply_markup: Union[
                                    None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def sendVenue(self, chat_id, latitude, longitude, title, address, foursquare_id=None, foursquare_type=None,
                  disable_notification=None, reply_to_message_id=None,
                  reply_markup: Union[
                      None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def sendContact(self, chat_id, phone_number, first_name, last_name=None, vcard=None, disable_notification=None,
                    reply_to_message_id=None,
                    reply_markup: Union[
                        None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def sendPoll(self, chat_id, question, options, is_anonymous=None, type=None, allows_multiple_answers=None,
                 correct_option_id=None, is_closed=None, disable_notification=None, reply_to_message_id=None,
                 reply_markup: Union[
                     None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:
        return self.request_and_result(create_params_from_args(locals()), Message)

    def sendChatAction(self, chat_id, action):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def getUserProfilePhotos(self, user_id, offset=None, limit=None):
        return self.request_and_result(create_params_from_args(locals()), UserProfilePhotos)

    def getFile(self, file_id):
        return self.request_and_result(create_params_from_args(locals()), File)

    def kickChatMember(self, chat_id, user_id, until_date=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def unbanChatMember(self, chat_id, user_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def restrictChatMember(self, chat_id, user_id, permissions, until_date=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def promoteChatMember(self, chat_id, user_id, can_change_info=None, can_post_messages=None, can_edit_messages=None,
                          can_delete_messages=None, can_invite_users=None, can_restrict_members=None,
                          can_pin_messages=None, can_promote_members=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def setChatAdministratorCustomTitle(self, chat_id, user_id, custom_title):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def setChatPermissions(self, chat_id, permissions):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def exportChatInviteLink(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def setChatPhoto(self, chat_id, photo):
        return self.request_and_result(create_params_from_args(locals(), ['photo']), bool, files={'photo': photo})

    def deleteChatPhoto(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def setChatTitle(self, chat_id, title):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def setChatDescription(self, chat_id, description=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def pinChatMessage(self, chat_id, message_id, disable_notification=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def unpinChatMessage(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def leaveChat(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def getChat(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), Chat)

    # TODO getChatAdministrators

    def getChatMembersCount(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), int)

    def getChatMember(self, chat_id, user_id):
        return self.request_and_result(create_params_from_args(locals()), ChatMember)

    def setChatStickerSet(self, chat_id, sticker_set_name):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def deleteChatStickerSet(self, chat_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def answerCallbackQuery(self, callback_query_id, text=None, show_alert=None, url=None, cache_time=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def editMessageText(self, text, chat_id=None, message_id=None, inline_message_id=None, parse_mode=None,
                        disable_webpage_preview=None,
                        reply_markup: Union[
                            None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def editMessageCaption(self, chat_id=None, message_id=None, inline_message_id=None, caption=None, parse_mode=None,
                           reply_markup: Union[
                               None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def editMessageMedia(self, media, chat_id=None, message_id=None, inline_message_id=None,
                         reply_markup: Union[
                             None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def editMessageReplyMarkup(self, chat_id=None, message_id=None, inline_message_id=None,
                               reply_markup: Union[
                                   None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def stopPoll(self, chat_id, message_id, reply_markup=None):
        return self.request_and_result(create_params_from_args(locals()), Poll)

    def deleteMessage(self, chat_id, message_id):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def sendSticker(self, chat_id, sticker, upload=False, disable_notification=None, reply_to_message_id=None,
                    reply_markup: Union[
                        None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), Message)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'sticker']),
                Message,
                files={'sticker': sticker}
            )

    def getStickerSet(self, name):
        return self.request_and_result(create_params_from_args(locals()), StickerSet)

    def uploadStickerFile(self, user_id, png_sticker):
        return self.request_and_result(create_params_from_args(locals(), ['png_sticker']), File,
                                       files={'png_sticker': png_sticker})

    def createNewStickerSet(self, user_id, name, title, png_sticker, emojis, upload=False, contains_masks=None,
                            mask_position=None):
        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), bool)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'png_sticker']),
                bool,
                files={'png_sticker': png_sticker}
            )

    def addStickerToSet(self, user_id, name, png_sticker, emojis, upload=False, mask_position=None):
        if not upload:
            return self.request_and_result(create_params_from_args(locals(), ['upload']), bool)
        else:
            return self.request_and_result(
                create_params_from_args(locals(), ['upload', 'png_sticker']),
                bool,
                files={'png_sticker': png_sticker}
            )

    def setStickerPositionInSet(self, sticker, position):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def deleteStickerFromSet(self, sticker):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def answerInlineQuery(self, inline_query_id, results, cache_time=None, is_personal=None, next_offset=None,
                          switch_pm_text=None, switch_pm_parameter=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def sendInvoice(self, chat_id, title, description, payload, provider_token, start_parameter, currency, prices,
                    provider_data=None, photo_url=None, photo_size=None, photo_width=None, photo_height=None,
                    need_name=None, need_phone_number=None, need_email=None, need_shipping_address=None,
                    send_phone_number_to_provider=None, send_email_to_provider=None, is_flexible=None,
                    disable_notification=None, reply_to_message_id=None,
                    reply_markup: Union[
                        None, InlineKeyboardMarkup, ReplyKeyboardMarkup, ReplyKeyboardRemove, ForceReply] = None) -> Message:

        return self.request_and_result(create_params_from_args(locals()), Message)

    def answerShippingQuery(self, shipping_query_id, ok, shipping_options=None, error_message=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def answerPreCheckoutQuery(self, pre_checkout_query_id, ok, error_message=None):
        return self.request_and_result(create_params_from_args(locals()), bool)

    def sendGame(self, chat_id, game_short_name, disable_notification=None, reply_to_message_id=None,
                 reply_markup=None):
        return self.request_and_result(create_params_from_args(locals()), Message)

    def setGameScore(self, user_id, score, force=None, disable_edit_message=None, chat_id=None, message_id=None,
                     inline_message_id=None):
        return self.request_and_result(create_params_from_args(locals()), Message)

    # TODO getGameHighScores



from typing import Optional

from . import BasicType
from . import inputmessagecontent, inlinekeyboardmarkup


class InlineQueryResult(BasicType):
    def __init__(self, obj=None):
        super(InlineQueryResult, self).__init__(obj)


class InlineQueryResultArticle(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'title': str,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        },
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'url': str,
        'hide_url': BasicType.bool_interpreter,
        'description': str,
        'thumb_url': str,
        'thumb_width': int,
        'thumb_height': int
    }

    def __init__(self, obj=None):
        super(InlineQueryResultArticle, self).__init__(obj)

    @classmethod
    def a(cls, id: str, title: str, input_message_content: inputmessagecontent.InputMessageContent,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None, url: Optional[str] = None,
          hide_url: Optional[bool] = None, description: Optional[str] = None, thumb_url: Optional[str] = None,
          thumb_width: Optional[int] = None, thumb_height: Optional[int] = None):
        type = 'article'
        return super().a(**locals())


class InlineQueryResultPhoto(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'photo_url': str,
        'thumb_url': str,
        'photo_width': int,
        'photo_height': int,
        'title': str,
        'description': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultPhoto, self).__init__(obj)

    @classmethod
    def a(cls, id: str, photo_url: str, thumb_url: str, photo_width: Optional[int] = None,
          photo_height: Optional[int] = None, title: Optional[str] = None, description: Optional[str] = None,
          caption: Optional[str] = None, parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'photo'
        return super().a(**locals())


class InlineQueryResultGif(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'gif_url': str,
        'gif_width': int,
        'gif_height': int,
        'gif_duration': int,
        'thumb_url': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultGif, self).__init__(obj)

    @classmethod
    def a(cls, id: str, gif_url: str, thumb_url: str, gif_width: Optional[int] = None, gif_height: Optional[int] = None,
          gif_duration: Optional[int] = None, title: Optional[str] = None, caption: Optional[str] = None,
          parse_mode: Optional[str] = None, input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'gif'
        return super().a(**locals())


class InlineQueryResultMpeg4Gif(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'mpeg4_url': str,
        'mpeg4_width': int,
        'mpeg4_height': int,
        'mpeg4_duration': int,
        'thumb_url': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultMpeg4Gif, self).__init__(obj)

    @classmethod
    def a(cls, id: str, mpeg4_url: str, thumb_url: str, mpeg4_width: Optional[int] = None,
          mpeg4_height: Optional[int] = None, mpeg4_duration: Optional[int] = None, title: Optional[str] = None,
          caption: Optional[str] = None, parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'mpeg4_gif'
        return super().a(**locals())


class InlineQueryResultVideo(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'video_url': str,
        'mime_type': str,
        'thumb_url': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'video_width': int,
        'video_height': int,
        'video_duration': int,
        'description': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultVideo, self).__init__(obj)

    @classmethod
    def a(cls, id: str, video_url: str, mime_type: str, thumb_url: str, title: str,
          caption: Optional[str] = None, parse_mode: Optional[str] = None, video_width: Optional[int] = None,
          video_height: Optional[int] = None, video_duration: Optional[int] = None, description: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'video'
        return super().a(**locals())


class InlineQueryResultAudio(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'audio_url': str,
        'title': str,
        'caption': str,
        'performer': str,
        'audio_duration': int,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultAudio, self).__init__(obj)

    @classmethod
    def a(cls, id: str, audio_url: str, title: str, caption: Optional[str] = None,
          parse_mode: Optional[str] = None, performer: Optional[str] = None, audio_duration: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'audio'
        return super().a(**locals())


class InlineQueryResultVoice(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'voice_url': str,
        'voice_duration': int,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultVoice, self).__init__(obj)

    @classmethod
    def a(cls, id: str, voice_url: str, title: str, caption: Optional[str] = None,
          parse_mode: Optional[str] = None, voice_duration: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'voice'
        return super().a(**locals())


class InlineQueryResultDocument(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'document_url': str,
        'mime_type': str,
        'description': str,
        'thumb_url': str,
        'thumb_width': int,
        'thumb_height': int,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultDocument, self).__init__(obj)

    @classmethod
    def a(cls, id: str, title: str, document_url: str, mime_type: str, caption: Optional[str] = None,
          parse_mode: Optional[str] = None, description: Optional[str] = None, thumb_url: Optional[str] = None,
          thumb_width: Optional[int] = None, thumb_height: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'document'
        return super().a(**locals())


class InlineQueryResultLocation(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'latitude': str,
        'longitude': str,
        'title': str,
        'live_period': int,
        'thumb_width': int,
        'thumb_height': int,
        'thumb_url': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultLocation, self).__init__(obj)

    @classmethod
    def a(cls, id: str, latitude: str, longitude: str, title: str, live_period: Optional[int] = None,
          thumb_url: Optional[str] = None, thumb_width: Optional[int] = None, thumb_height: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'document'
        return super().a(**locals())


class InlineQueryResultVenue(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'latitude': str,
        'longitude': str,
        'title': str,
        'address': str,
        'foursquare_id': str,
        'foursquare_type': str,
        'thumb_url': str,
        'thumb_width': int,
        'thumb_height': int,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultVenue, self).__init__(obj)

    @classmethod
    def a(cls, id: str, latitude: str, longitude: str, title: str, address: str,
          foursquare_id: Optional[str] = None, foursqure_type: Optional[str] = None, thumb_url: Optional[str] = None,
          thumb_width: Optional[int] = None, thumb_height: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'venue'
        return super().a(**locals())


class InlineQueryResultContact(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'phone_number': str,
        'first_name': str,
        'last_name': str,
        'vcard': str,
        'thumb_url': str,
        'thumb_width': int,
        'thumb_height': int,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultContact, self).__init__(obj)

    @classmethod
    def a(cls, id: str, phone_number: str, first_name: str, last_name: Optional[str] = None,
          vcard: Optional[str] = None, thumb_url: Optional[str] = None, thumb_width: Optional[int] = None,
          thumb_height: Optional[int] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'contact'
        return super().a(**locals())


class InlineQueryResultGame(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'game_short_name': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
    }

    def __init__(self, obj=None):
        super(InlineQueryResultGame, self).__init__(obj)

    @classmethod
    def a(cls, id: str, game_short_name: str,
        reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'game'
        return super().a(**locals())


class InlineQueryResultCachedPhoto(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'photo_file_id': str,
        'description': str,
        'thumb_url': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedPhoto, self).__init__(obj)

    @classmethod
    def a(cls, id: str, photo_file_id: str, title: Optional[str] = None,
          description: Optional[str] = None, caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'photo'
        return super().a(**locals())


class InlineQueryResultCachedGif(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'gif_file_id': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedGif, self).__init__(obj)

    @classmethod
    def a(cls, id: str, gif_file_id: str, title: Optional[str] = None,
          caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'gif'
        return super().a(**locals())


class InlineQueryResultCachedMpeg4Gif(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'mpeg4_file_id': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedMpeg4Gif, self).__init__(obj)

    @classmethod
    def a(cls, id: str, mpeg4_file_id: str, title: Optional[str] = None,
          caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'mpeg4_gif'
        return super().a(**locals())


class InlineQueryResultCachedSticker(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'sticker_file_id': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedSticker, self).__init__(obj)

    @classmethod
    def a(cls, id: str, sticker_file_id: str,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'sticker'
        return super().a(**locals())


class InlineQueryResultCachedDocument(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'title': str,
        'caption': str,
        'description': str,
        'document_file_id': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedDocument, self).__init__(obj)

    @classmethod
    def a(cls, id: str, document_file_id: str, title: str,
          description: Optional[str] = None, caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'document'
        return super().a(**locals())


class InlineQueryResultCachedVideo(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'video_file_id': str,
        'title': str,
        'caption': str,
        'description': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedVideo, self).__init__(obj)

    @classmethod
    def a(cls, id: str, video_file_id: str, title: str,
          description: Optional[str] = None, caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'video'
        return super().a(**locals())


class InlineQueryResultCachedVoice(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'voice_file_id': str,
        'title': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedVoice, self).__init__(obj)

    @classmethod
    def a(cls, id: str, voice_file_id: str, title: str,
          caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'voice'
        return super().a(**locals())


class InlineQueryResultCachedAudio(InlineQueryResult):
    fields = {
        'type': str,
        'id': str,
        'audio_file_id': str,
        'caption': str,
        'parse_mode': str,
        'reply_markup': inlinekeyboardmarkup.InlineKeyboardMarkup,
        'input_message_content': {
            'class': inputmessagecontent.InputMessageContent,
            'validation': False
        }
    }

    def __init__(self, obj=None):
        super(InlineQueryResultCachedAudio, self).__init__(obj)

    @classmethod
    def a(cls, id: str, audio_file_id: str,
          caption: Optional[str] = None,
          parse_mode: Optional[str] = None,
          input_message_content: Optional[inputmessagecontent.InputMessageContent] = None,
          reply_markup: Optional[inlinekeyboardmarkup.InlineKeyboardMarkup] = None):
        type = 'audio'
        return super().a(**locals())
